package example.localplugin.coolemall.computebox2;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import schedframe.events.scheduling.SchedulingEvent;
import schedframe.exceptions.ResourceException;
import schedframe.resources.StandardResourceType;
import schedframe.resources.computing.ComputingResource;
import schedframe.resources.computing.Processor;
import schedframe.resources.computing.coolemall.Node;
import schedframe.resources.units.ProcessingElements;
import schedframe.resources.units.ResourceUnit;
import schedframe.resources.units.ResourceUnitName;
import schedframe.resources.units.StandardResourceUnitName;
import schedframe.scheduling.manager.resources.ClusterResourceManager;
import schedframe.scheduling.manager.resources.ResourceManager;
import schedframe.scheduling.manager.tasks.JobRegistry;
import schedframe.scheduling.manager.tasks.JobRegistryImpl;
import schedframe.scheduling.plan.SchedulingPlanInterface;
import schedframe.scheduling.plan.impl.SchedulingPlan;
import schedframe.scheduling.plugin.grid.ModuleList;
import schedframe.scheduling.queue.TaskQueue;
import schedframe.scheduling.queue.TaskQueueList;
import schedframe.scheduling.tasks.TaskInterface;
import schedframe.scheduling.tasks.phases.PhaseBehaviour;
import testbed_hpc.Node_Fan_Mapping;
import dcworms.schedframe.scheduling.ExecTask;
import example.energy.coolemall.CoolEmAllTestbedMeasurements;
import example.localplugin.BaseLocalSchedulingPlugin;
import gridsim.dcworms.DCWormsTags;

public class CB2_FCFS_Random_SP extends BaseLocalSchedulingPlugin {

	private Random rand;
	
	public CB2_FCFS_Random_SP () {
		rand = new Random(47);
		Node_Fan_Mapping.init();
	}

	public SchedulingPlanInterface<?> schedule(SchedulingEvent event, TaskQueueList queues, JobRegistry jobRegistry,
			 ResourceManager resManager, ModuleList modules) {

		ClusterResourceManager resourceManager = (ClusterResourceManager) resManager;
		SchedulingPlan plan = new SchedulingPlan();
		// choose the events types to serve.
		// Different actions for different events are possible.
		switch (event.getType()) {
		case START_TASK_EXECUTION:
		case TASK_FINISHED:
		//case TIMER:
			// our tasks are placed only in first queue (see
			// BaseLocalSchedulingPlugin.placeJobsInQueues() method)
			TaskQueue q = queues.get(0);
			
			// check all tasks in queue
			for (int i = 0; i < q.size(); i++) {
				TaskInterface<?> task = q.get(i);
				// if status of the tasks in READY
				if (task.getStatus() == DCWormsTags.READY) {

					Map<ResourceUnitName, ResourceUnit> choosenResources = chooseResourcesForExecution(resourceManager, task);
					if (choosenResources != null) {
						addToSchedulingPlan(plan, task, choosenResources);
					}
				}
			}
			break;
		}
		return plan;
	}

	@SuppressWarnings("unchecked")
	private Map<ResourceUnitName, ResourceUnit> chooseResourcesForExecution(
			ClusterResourceManager resourceManager, TaskInterface<?> task) {

		Map<ResourceUnitName, ResourceUnit> map = new HashMap<ResourceUnitName, ResourceUnit>();
		
		List<Node> nodes = null;
		try {
			nodes = (List<Node>) resourceManager.getResourcesOfType(StandardResourceType.Node);
		} catch (ResourceException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		List<Node> avNodes = filterNodes(nodes, task);
		if(avNodes.size() == 0)
			return null;
		Node node = randomNode(avNodes);
		int cpuRequest;
		try {
			cpuRequest = Double.valueOf(task.getCpuCntRequest()).intValue();
		} catch (NoSuchFieldException e) {
			cpuRequest = 0;
		}

		if (cpuRequest != 0) {

			List<ComputingResource> choosenResources = new ArrayList<ComputingResource>();	
			
			choosenResources.add(node.getProcessors().get(0));
			ProcessingElements pe = new ProcessingElements();
			pe.addAll(choosenResources);
			map.put(StandardResourceUnitName.PE, pe);
			return map;
		}
		return null;
	}
	
	private List<Node> filterNodes(List<Node> nodes, TaskInterface<?> task){
		List<Node> filteredNodes = new ArrayList<Node>();
		int cpuRequest;
		try {
			cpuRequest = Double.valueOf(task.getCpuCntRequest()).intValue();
		} catch (NoSuchFieldException e) {
			cpuRequest = 0;
		}
		for (Node node : nodes) {

			if (cpuRequest != 0) {

				double sumLoad = 0;
				Processor proc = node.getProcessors().get(0);
				JobRegistry jr = new JobRegistryImpl(proc.getFullName());
				for(ExecTask execTask: jr.getRunningTasks()){
					double cpuUsage = 100;
					double nrOfThreadsOfApplciation = 1;
					for(PhaseBehaviour pb: execTask.getCurrentResourceConsumption().getBehaviourList()){
						if(pb.getResouceName().equals("PM_CPU_Usage")){
							cpuUsage = pb.getUtilization();
						}
						if(pb.getResouceName().equals("PM_Threads")){
							nrOfThreadsOfApplciation = pb.getUtilization();
							break;
						}
					}
					sumLoad = sumLoad + nrOfThreadsOfApplciation;
				}

				int nrOfThreadsOnCpu = 1;
				
				if(proc.getResourceCharacteristic().getParameters().get("product") != null){
					String productName = proc.getResourceCharacteristic().getParameters().get("product").get(0).getContent();
					if(productName.equals("Fusion G - T40N"))
						nrOfThreadsOnCpu = CoolEmAllTestbedMeasurements.Fusion_G_T40N_NR_OF_THREADS;
					else if(productName.equals("Atom - D510"))
						nrOfThreadsOnCpu = CoolEmAllTestbedMeasurements.Atom_D510_NR_OF_THREADS/2;
					else if(productName.equals("Atom - N2600"))
						nrOfThreadsOnCpu = CoolEmAllTestbedMeasurements.Atom_N2600_NR_OF_THREADS/2;
					else if(productName.equals("Core i7 - 2715QE"))
						nrOfThreadsOnCpu = CoolEmAllTestbedMeasurements.Core_i7_2715QE_NR_OF_THREADS/2;
					else if(productName.equals("Core i7 - 3615QE"))
						nrOfThreadsOnCpu = CoolEmAllTestbedMeasurements.Core_i7_3615QE_NR_OF_THREADS/2;
				}

				System.out.println(sumLoad + ":" + nrOfThreadsOnCpu + ";" + cpuRequest);
				int freeThreads = (int) (nrOfThreadsOnCpu - sumLoad) ;

				
				if(freeThreads < cpuRequest)
					continue;
				
				filteredNodes.add(node);
			}
		}
		
		return filteredNodes;
	}
	
	private Node randomNode(List<Node> nodes){
		return nodes.get(rand.nextInt(nodes.size()));
	}

	
}

